==> smasher-1.1.0-source.tar.gz: smasher-1.1.0/src/monochromegrid.cpp <==
#include <wx/gdicmn.h>
#include <wx/tokenzr.h>
#include "monochromegrid.h"

BEGIN_EVENT_TABLE(MonochromeGrid, wxGrid)
    EVT_GRID_CELL_LEFT_CLICK( MonochromeGrid::OnLeftClick )
    EVT_GRID_CELL_LEFT_DCLICK( MonochromeGrid::OnLeftDClick )
    EVT_GRID_LABEL_LEFT_CLICK( MonochromeGrid::OnLabelLeftClick )
    EVT_GRID_LABEL_RIGHT_CLICK( MonochromeGrid::OnLabelRightClick )
    EVT_GRID_RANGE_SELECT( MonochromeGrid::OnGridRangeSelect )
END_EVENT_TABLE()

MonochromeGrid::MonochromeGrid (
    wxWindow *parent,
    int id,
    size_t rowNo,
    size_t colNo,
    int cellHeightParameter,
    int cellWidthParameter,
    bool verticalConstraintParameter ) :
        wxGrid ( parent, id, wxDefaultPosition, wxDefaultSize, 0 )
{
    EnableEditing ( false );
    EnableDragCell ( false );
    EnableDragColMove ( false );
    EnableDragColSize ( false );
    EnableDragGridSize ( false );
    EnableDragRowSize ( false );
    SetSelectionForeground ( *wxLIGHT_GREY );
    SetSelectionBackground ( *wxLIGHT_GREY );
    SetColLabelTextOrientation ( wxHORIZONTAL );
    SetLabelTextColour( wxTheColourDatabase->Find ( _T ( "GREY" ) ) );
    SetGridLineColour( *wxLIGHT_GREY );

    CreateGrid ( rowNo, colNo );

    SetDefaultRowSize ( cellHeightParameter, true );
    SetDefaultColSize ( ( cellWidthParameter == -1) ? ( int ) ( 16.0/9.0 * cellHeightParameter ) : cellWidthParameter, true );
#ifdef __WXMSW__
    SetRowLabelSize ( 86 );
#else
    SetRowLabelSize ( 96 );
#endif
    SetColLabelSize ( GetDefaultRowSize( ) );
    verticalConstraint = verticalConstraintParameter;

    for ( size_t i = 0; i < colNo; i++ )
    {
        SetColSize ( i, cellWidth );
    }
    for ( size_t i = 0; i < rowNo; i++ )
    {
        SetRowSize ( i, GetDefaultRowSize () );
    }

    SetRowMinimalAcceptableHeight ( 0 );
    SetColMinimalAcceptableWidth ( 0 );
}

void MonochromeGrid::autoLabel (  )
{
    wxString s;
    for ( int i = 0; i < GetNumberRows(); i++ )
    {
        s.Printf ( _("%i"), i+1 );
        SetRowLabelValue ( i, s );
    }
    for ( int i = 0; i < GetNumberCols(); i++ )
    {
        s.Printf ( _("%i"), i+1 );
        SetColLabelValue ( i, s );
    }
}

void MonochromeGrid::clearAll ( )
{
    BeginBatch ( );
    int rowCount, colCount;
    rowCount = GetNumberRows ( );
    colCount = GetNumberCols ( ) ;
    for ( int i = 0; i < colCount; i++ )
    {
        for ( int j = 0; j < rowCount; j++ )
        {
            setValue ( false, j, i );
        }
    }
    EndBatch ();
}

void MonochromeGrid::clearRow ( int row )
{
    BeginBatch ( );
    if ( row > GetNumberRows() )
        return;
    int colCount;
    colCount = GetNumberCols ( ) ;
    for ( int i = 0; i < colCount; i++ )
    {
        setValue ( false, row, i );
    }
    EndBatch ();
}

void MonochromeGrid::clearCol ( int col )
{
    BeginBatch ();
    if ( col > GetNumberCols() )
        return;
    int rowCount;
    rowCount = GetNumberRows ( ) ;
    for ( int i = 0; i < rowCount; i++ )
    {
        setValue ( false, i, col );
    }
    EndBatch ();
}


void MonochromeGrid::getPositionVector ( std::vector<size_t> &positionVector )
{
    bool sliceIsSelected;
    int rowCount, colCount;
    rowCount = GetNumberRows ( );
    colCount = GetNumberCols ( ) ;
    for ( int col = 0; col < colCount; col++ )
    {
        sliceIsSelected = false;
        for ( int row = 0; row < rowCount; row++ )
        {
            if ( isSelected ( row, col ) )
            {
                positionVector.push_back ( ( size_t ) row );
                sliceIsSelected = true;
                break;
            }
        }
        if ( !sliceIsSelected )
            positionVector.push_back ( UINT_MAX );
    }
}

bool MonochromeGrid::isSelected ( int row, int col )
{
    wxString colourString = GetCellBackgroundColour( row, col ).GetAsString();
    if ( colourString != wxWHITE->GetAsString() )
        return true;
    return false;
}

void MonochromeGrid::loadState ( const wxString& state )
{
    BeginBatch ();
    std::string stdState = ( const char *) state.mb_str ( wxConvUTF8 );

    int col, row;
    col = row = 0;
    for ( size_t i = 0; i < stdState.size(); i++)
    {
        if ( stdState.at ( i ) == '#' )
        {
            row = 0;
            col++;
        }
        else
        {
            setValue ( ( stdState.at ( i ) == '1' ) ? true : false, row, col );
            row ++;
        }
    }
    EndBatch ();
}

void MonochromeGrid::OnGridRangeSelect(wxGridRangeSelectEvent& event)
{
    BeginBatch();
    int x1, y1, x2, y2;
    x1 = event.GetLeftCol();
    y1 = event.GetTopRow();
    x2 = event.GetRightCol();
    y2 = event.GetBottomRow();

    if ( ( x1 == 0 ) && ( y1 == 0 ) && ( x2 == GetNumberCols() - 1 ) && ( y2 == GetNumberRows() - 1 ) )
    {
        event.Skip();
        EndBatch();
        return;
    }

    int i, j;

    if ( verticalConstraint )
    {
        for ( i = x1; i <= x2; i++ )
            clearCol( i );

        if ( ( x2 - x1 ) == ( y2 - y1 ) )
        {
            for ( i = x1, j = y1; i <= x2 && j <= y2; i++, j++ )
                setValue ( true, j, i );
        }
        else
        {
            for ( i= x1; i <= x2; i++ )
                setValue ( true, y1, i );
        }
    }
    else
    {
        for ( i = x1; i <= x2; i++ )
        {
            for ( j = y1; j <= y2; j++ )
                setValue ( true, j, i );
        }
    }
    ClearSelection();
    event.Skip();
    EndBatch ();
}


void MonochromeGrid::OnLabelLeftClick ( wxGridEvent &e )
{
    int row;
    row = e.GetRow();
    if ( row == -1 )
    {
        e.Skip();
        return;
    }

    BeginBatch();
    if ( verticalConstraint )
        clearAll();
    for ( int i = 0; i < GetNumberCols(); i++ )
        setValue ( true, row, i );
    EndBatch();
}

void MonochromeGrid::OnLabelRightClick ( wxGridEvent &e )
{
    BeginBatch();
    int row;
    row = e.GetRow();
    if ( row == -1 )
        return;
    for ( int i = 0; i < GetNumberCols(); i++ )
        setValue ( false, row, i );
    EndBatch();
}

void MonochromeGrid::OnLeftClick ( wxGridEvent &e )
{
    BeginBatch ( );
    int col, row, rowCount, colCount;
    col = e.GetCol ( );
    row = e.GetRow ( );

    rowCount = GetNumberRows();
    colCount = GetNumberCols();

    for ( int i = 0; (i < rowCount) && verticalConstraint; i++)
    {
        if ( isSelected ( i, col ) )
        {
            if ( i!= row )
            {
                setValue ( false, i, col );
                break;
            }
        }
    }

    toggleValue ( row, col );
    e.Skip();
    EndBatch ( );
}

void MonochromeGrid::OnLeftDClick ( wxGridEvent &e )
{
    return;
}

void MonochromeGrid::resizeCols ( int i )
{
    BeginBatch();
    int colCount, j;
    colCount = GetNumberCols ( ) ;
    if ( i > colCount)
        AppendCols ( i - colCount );
    else
    {
        j = colCount - i;
        DeleteCols ( colCount - j, j );
    }
    EndBatch();
}

void MonochromeGrid::resizeRows ( int i )
{
    BeginBatch();
    int rowCount, j;
    rowCount = GetNumberRows ( );
    if ( i > rowCount )
        AppendRows ( i - rowCount );
    else
    {
        j = rowCount - i;
        DeleteRows ( rowCount - j, j );
    }
    EndBatch();
}

void MonochromeGrid::rotateLeft ( )
{
    BeginBatch();

    wxString state = saveState();

    wxStringTokenizer tokenizer( state, _T("#") );

    std::vector<wxString> columns;

    while ( tokenizer.HasMoreTokens() )
    {
        columns.push_back ( tokenizer.GetNextToken() );
    }

    size_t colSize = columns.size();
    state.clear ();

    for ( size_t i = 1; i < colSize; i++ )
    {
        state += columns.at ( i );
        state += _( "#" );
    }

    state += columns.at ( 0 );

    clearAll ();
    loadState ( state );
    EndBatch();
}

void MonochromeGrid::rotateRight ( )
{
    BeginBatch();

    wxString state = saveState();

    wxStringTokenizer tokenizer( state, _T("#") );

    std::vector<wxString> columns;

    while ( tokenizer.HasMoreTokens() )
    {
        columns.push_back ( tokenizer.GetNextToken() );
    }

    size_t colSize = columns.size();
    state.clear ();

    state += columns.at ( colSize - 1 );
    state += _( "#" );

    for ( size_t i = 0; i < colSize - 1; i++ )
    {
        state += columns.at ( i );
        state += _( "#" );
    }

    clearAll ();
    loadState ( state );
    EndBatch();
}

wxString MonochromeGrid::saveState ()
{
    std::string stdState;

    int colNo, rowNo;
    colNo = GetNumberCols();
    rowNo = GetNumberRows();
    for ( int col = 0; col < colNo; col++)
    {
        for ( int row = 0; row < rowNo; row++)
        {
            stdState.append( (isSelected ( row, col )) ? "1" : "0" );
        }
        stdState.append ( "#" );
    }
    wxString ret = wxString ( stdState.c_str(), wxConvUTF8, stdState.size());
    return ret;
}

void MonochromeGrid::setPositionVector ( const std::vector<size_t>& positionVector )
{
    BeginBatch ( );
    clearAll ( );

    size_t colCount = (size_t) GetNumberCols ( ) ;

    if ( ( positionVector.size() > colCount ) && verticalConstraint )
        return;

    for ( int col = 0; col < (int) positionVector.size (); col++ )
    {
        setValue ( true, positionVector.at ( col ), col );
    }

    EndBatch ();
}

void MonochromeGrid::setValue ( bool value, int row, int col )
{
    BeginBatch ( );
    SetCellBackgroundColour( row, col, ( value ) ? *wxBLACK : *wxWHITE );
    EndBatch ();
}

void MonochromeGrid::showRow ( const wxString& rowLabelText, bool show )
{
    BeginBatch();

    int row, rowNo;
    row = -1;
    rowNo = GetNumberRows();
    for ( int i = 0; i < rowNo; i++ )
    {
        if ( GetRowLabelValue ( i ) == rowLabelText )
        {
            row = i;
            break;
        }
    }

    if ( row == -1 )
        return;

    SetRowSize ( row, ( show ) ? GetDefaultRowSize () : 0 );
    EndBatch();
}

void MonochromeGrid::toggleValue ( int row, int col )
{
    BeginBatch();
    if ( isSelected ( row, col ) )
        setValue( false, row, col);
    else
        setValue( true, row, col);
    EndBatch();
}

==> smasher-1.1.6-source.tar.gz: smasher-1.1.6/src/monochromegrid.cpp <==
#include <wx/gdicmn.h>
#include <wx/tokenzr.h>
#include "monochromegrid.h"

BEGIN_EVENT_TABLE(MonochromeGrid, wxGrid)
    EVT_GRID_CELL_LEFT_CLICK( MonochromeGrid::OnLeftClick )
    EVT_GRID_CELL_LEFT_DCLICK( MonochromeGrid::OnLeftDClick )
    EVT_GRID_LABEL_LEFT_CLICK( MonochromeGrid::OnLabelLeftClick )
    EVT_GRID_LABEL_RIGHT_CLICK( MonochromeGrid::OnLabelRightClick )
    EVT_GRID_RANGE_SELECT( MonochromeGrid::OnGridRangeSelect )
END_EVENT_TABLE()

MonochromeGrid::MonochromeGrid (
    wxWindow *parent,
    int id,
    size_t rowNo,
    size_t colNo,
    int cellHeightParameter,
    int cellWidthParameter,
    bool verticalConstraintParameter ) :
        wxGrid ( parent, id, wxDefaultPosition, wxDefaultSize, 0 )
{
    EnableEditing ( false );
    EnableDragCell ( false );
    EnableDragColMove ( false );
    EnableDragColSize ( false );
    EnableDragGridSize ( false );
    EnableDragRowSize ( false );
    SetSelectionForeground ( *wxLIGHT_GREY );
    SetSelectionBackground ( *wxLIGHT_GREY );
    SetColLabelTextOrientation ( wxHORIZONTAL );
    SetLabelTextColour( wxTheColourDatabase->Find ( _T ( "GREY" ) ) );
    SetGridLineColour( *wxLIGHT_GREY );

    CreateGrid ( rowNo, colNo );

    SetDefaultRowSize ( cellHeightParameter, true );
    SetDefaultColSize ( ( cellWidthParameter == -1) ? ( int ) ( 16.0/9.0 * cellHeightParameter ) : cellWidthParameter, true );
#ifdef __WXMSW__
    SetRowLabelSize ( 86 );
#else
    SetRowLabelSize ( 96 );
#endif
    SetColLabelSize ( GetDefaultRowSize( ) );
    verticalConstraint = verticalConstraintParameter;

    for ( size_t i = 0; i < colNo; i++ )
    {
        SetColSize ( i, GetDefaultColSize () );//eal
    }
    for ( size_t i = 0; i < rowNo; i++ )
    {
        SetRowSize ( i, GetDefaultRowSize () );
    }

    SetRowMinimalAcceptableHeight ( 0 );
    SetColMinimalAcceptableWidth ( 0 );
}

void MonochromeGrid::autoLabel (  )
{
    wxString s;
    for ( int i = 0; i < GetNumberRows(); i++ )
    {
        s.Printf ( _("%i"), i+1 );
        SetRowLabelValue ( i, s );
    }
    for ( int i = 0; i < GetNumberCols(); i++ )
    {
        s.Printf ( _("%i"), i+1 );
        SetColLabelValue ( i, s );
    }
}

void MonochromeGrid::clearAll ( )
{
    BeginBatch ( );
    int rowCount, colCount;
    rowCount = GetNumberRows ( );
    colCount = GetNumberCols ( ) ;
    for ( int i = 0; i < colCount; i++ )
    {
        for ( int j = 0; j < rowCount; j++ )
        {
            setValue ( false, j, i );
        }
    }
    EndBatch ();
}

void MonochromeGrid::clearRow ( int row )
{
    BeginBatch ( );
    if ( row > GetNumberRows() )
        return;
    int colCount;
    colCount = GetNumberCols ( ) ;
    for ( int i = 0; i < colCount; i++ )
    {
        setValue ( false, row, i );
    }
    EndBatch ();
}

void MonochromeGrid::clearCol ( int col )
{
    BeginBatch ();
    if ( col > GetNumberCols() )
        return;
    int rowCount;
    rowCount = GetNumberRows ( ) ;
    for ( int i = 0; i < rowCount; i++ )
    {
        setValue ( false, i, col );
    }
    EndBatch ();
}


void MonochromeGrid::getPositionVector ( std::vector<size_t> &positionVector )
{
    bool sliceIsSelected;
    int rowCount, colCount;
    rowCount = GetNumberRows ( );
    colCount = GetNumberCols ( ) ;
    for ( int col = 0; col < colCount; col++ )
    {
        sliceIsSelected = false;
        for ( int row = 0; row < rowCount; row++ )
        {
            if ( isSelected ( row, col ) )
            {
                positionVector.push_back ( ( size_t ) row );
                sliceIsSelected = true;
                break;
            }
        }
        if ( !sliceIsSelected )
            positionVector.push_back ( UINT_MAX );
    }
}

bool MonochromeGrid::isSelected ( int row, int col )
{
    wxString colourString = GetCellBackgroundColour( row, col ).GetAsString();
    if ( colourString != wxWHITE->GetAsString() )
        return true;
    return false;
}

void MonochromeGrid::loadState ( const wxString& state )
{
    BeginBatch ();
    std::string stdState = ( const char *) state.mb_str ( wxConvUTF8 );

    int col, row;
    col = row = 0;
    for ( size_t i = 0; i < stdState.size(); i++)
    {
        if ( stdState.at ( i ) == '#' )
        {
            row = 0;
            col++;
        }
        else
        {
            setValue ( ( stdState.at ( i ) == '1' ) ? true : false, row, col );
            row ++;
        }
    }
    EndBatch ();
}

void MonochromeGrid::OnGridRangeSelect(wxGridRangeSelectEvent& event)
{
    BeginBatch();
    int x1, y1, x2, y2;
    x1 = event.GetLeftCol();
    y1 = event.GetTopRow();
    x2 = event.GetRightCol();
    y2 = event.GetBottomRow();

    if ( ( x1 == 0 ) && ( y1 == 0 ) && ( x2 == GetNumberCols() - 1 ) && ( y2 == GetNumberRows() - 1 ) )
    {
        event.Skip();
        EndBatch();
        return;
    }

    int i, j;

    if ( verticalConstraint )
    {
        for ( i = x1; i <= x2; i++ )
            clearCol( i );

        if ( ( x2 - x1 ) == ( y2 - y1 ) )
        {
            for ( i = x1, j = y1; i <= x2 && j <= y2; i++, j++ )
                setValue ( true, j, i );
        }
        else
        {
            for ( i= x1; i <= x2; i++ )
                setValue ( true, y1, i );
        }
    }
    else
    {
        for ( i = x1; i <= x2; i++ )
        {
            for ( j = y1; j <= y2; j++ )
                setValue ( true, j, i );
        }
    }
    ClearSelection();
    event.Skip();
    EndBatch ();
}


void MonochromeGrid::OnLabelLeftClick ( wxGridEvent &e )
{
    int row;
    row = e.GetRow();
    if ( row == -1 )
    {
        e.Skip();
        return;
    }

    BeginBatch();
    if ( verticalConstraint )
        clearAll();
    for ( int i = 0; i < GetNumberCols(); i++ )
        setValue ( true, row, i );
    EndBatch();
}

void MonochromeGrid::OnLabelRightClick ( wxGridEvent &e )
{
    BeginBatch();
    int row;
    row = e.GetRow();
    if ( row == -1 )
        return;
    for ( int i = 0; i < GetNumberCols(); i++ )
        setValue ( false, row, i );
    EndBatch();
}

void MonochromeGrid::OnLeftClick ( wxGridEvent &e )
{
    BeginBatch ( );
    int col, row, rowCount, colCount;
    col = e.GetCol ( );
    row = e.GetRow ( );

    rowCount = GetNumberRows();
    colCount = GetNumberCols();

    for ( int i = 0; (i < rowCount) && verticalConstraint; i++)
    {
        if ( isSelected ( i, col ) )
        {
            if ( i!= row )
            {
                setValue ( false, i, col );
                break;
            }
        }
    }

    toggleValue ( row, col );
    e.Skip();
    EndBatch ( );
}

void MonochromeGrid::OnLeftDClick ( wxGridEvent &e )
{
    return;
}

void MonochromeGrid::resizeCols ( int i )
{
    BeginBatch();
    int colCount, j;
    colCount = GetNumberCols ( ) ;
    if ( i > colCount)
        AppendCols ( i - colCount );
    else
    {
        j = colCount - i;
        DeleteCols ( colCount - j, j );
    }
    EndBatch();
}

void MonochromeGrid::resizeRows ( int i )
{
    BeginBatch();
    int rowCount, j;
    rowCount = GetNumberRows ( );
    if ( i > rowCount )
        AppendRows ( i - rowCount );
    else
    {
        j = rowCount - i;
        DeleteRows ( rowCount - j, j );
    }
    EndBatch();
}

void MonochromeGrid::rotateLeft ( )
{
    BeginBatch();

    wxString state = saveState();

    wxStringTokenizer tokenizer( state, _T("#") );

    std::vector<wxString> columns;

    while ( tokenizer.HasMoreTokens() )
    {
        columns.push_back ( tokenizer.GetNextToken() );
    }

    size_t colSize = columns.size();
    state.clear ();

    for ( size_t i = 1; i < colSize; i++ )
    {
        state += columns.at ( i );
        state += _( "#" );
    }

    state += columns.at ( 0 );

    clearAll ();
    loadState ( state );
    EndBatch();
}

void MonochromeGrid::rotateRight ( )
{
    BeginBatch();

    wxString state = saveState();

    wxStringTokenizer tokenizer( state, _T("#") );

    std::vector<wxString> columns;

    while ( tokenizer.HasMoreTokens() )
    {
        columns.push_back ( tokenizer.GetNextToken() );
    }

    size_t colSize = columns.size();
    state.clear ();

    state += columns.at ( colSize - 1 );
    state += _( "#" );

    for ( size_t i = 0; i < colSize - 1; i++ )
    {
        state += columns.at ( i );
        state += _( "#" );
    }

    clearAll ();
    loadState ( state );
    EndBatch();
}

wxString MonochromeGrid::saveState ()
{
    std::string stdState;

    int colNo, rowNo;
    colNo = GetNumberCols();
    rowNo = GetNumberRows();
    for ( int col = 0; col < colNo; col++)
    {
        for ( int row = 0; row < rowNo; row++)
        {
            stdState.append( (isSelected ( row, col )) ? "1" : "0" );
        }
        stdState.append ( "#" );
    }
    wxString ret = wxString ( stdState.c_str(), wxConvUTF8, stdState.size());
    return ret;
}

void MonochromeGrid::setPositionVector ( const std::vector<size_t>& positionVector )
{
    BeginBatch ( );
    clearAll ( );

    size_t colCount = (size_t) GetNumberCols ( ) ;

    if ( ( positionVector.size() > colCount ) && verticalConstraint )
        return;

    for ( int col = 0; col < (int) positionVector.size (); col++ )
    {
        setValue ( true, positionVector.at ( col ), col );
    }

    EndBatch ();
}

void MonochromeGrid::setValue ( bool value, int row, int col )
{
    BeginBatch ( );
    SetCellBackgroundColour( row, col, ( value ) ? *wxBLACK : *wxWHITE );
    EndBatch ();
}

void MonochromeGrid::showRow ( const wxString& rowLabelText, bool show )
{
    BeginBatch();

    int row, rowNo;
    row = -1;
    rowNo = GetNumberRows();
    for ( int i = 0; i < rowNo; i++ )
    {
        if ( GetRowLabelValue ( i ) == rowLabelText )
        {
            row = i;
            break;
        }
    }

    if ( row == -1 )
        return;

    SetRowSize ( row, ( show ) ? GetDefaultRowSize () : 0 );
    EndBatch();
}

void MonochromeGrid::toggleValue ( int row, int col )
{
    BeginBatch();
    if ( isSelected ( row, col ) )
        setValue( false, row, col);
    else
        setValue( true, row, col);
    EndBatch();
}
